@@ -25,6 +25,9 @@
 #include "general.h"
 #include "jump.h"
 #include "other.h"
+#ifdef USE_GDKPIXBUF
+#include "print.h"
+#endif
 #include "prop.h"
 #include "replace.h"
 #include "repinfo.h"
@@ -68,10 +71,15 @@ void command_open(gpointer callback_data,guint callback_action,
 	gtk_signal_connect_object(
 						GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
 							"clicked",gtk_widget_destroy,GTK_OBJECT(dialog));
+	if (openpath!=NULL)
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION(dialog),openpath);
 	gtk_widget_show(dialog);
 	gtk_grab_add(dialog);
 	gtk_main();
 	if (command_open_file!=NULL) {
+		g_free(openpath);
+		openpath=get_full_path(command_open_file);
+		*g_basename(openpath)='\0';
 		open_edit_file(command_open_file);
 		g_free(command_open_file);
 	}
@@ -123,8 +131,8 @@ void command_saveas(gpointer callback_data,guint callback_action,
 	gtk_signal_connect_object(
 						GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
 							"clicked",gtk_widget_destroy,GTK_OBJECT(dialog));
-	if (!ptw->create)
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(dialog),ptw->file);
+	if (savepath!=NULL)
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION(dialog),savepath);
 	gtk_widget_show(dialog);
 	gtk_grab_add(dialog);
 	gtk_main();
@@ -136,10 +144,14 @@ void command_saveas(gpointer callback_data,guint callback_action,
 			|| message_box("Text maid",text,1,
 										_("Ϥ(_Y)"),_("(_N)"),NULL)==0)
 									&& save_text_file(command_open_file,ptw)) {
+			g_free(savepath);
+			savepath=get_full_path(command_open_file);
+			*g_basename(savepath)='\0';
 			/* ե̾Ĵ٤ */
 			delete_edit_file(ptw->file);
 			ptw->file[0]='\0';
 			label=add_edit_file(command_open_file,&ptw->same);
+			gtk_label_set_text(GTK_LABEL(ptw->label),command_open_file);
 			gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
 																child,label);
 			g_free(label);
@@ -207,11 +219,11 @@ void command_reload(gpointer callback_data,guint callback_action,
 		ptw->edit=FALSE;
 		set_scroll_bar(ptw->hscroll,signal_value_changed_hscroll,ptw,
 					0,get_width_max(ptw),
-					MAX(ptw->drawing->allocation.width/ptw->fontsize,1),
+					MAX(ptw->drawing->allocation.width/ptw->fontwidth,1),
 																ptw->top.x);
 		set_scroll_bar(ptw->vscroll,signal_value_changed_vscroll,ptw,
 					0,ptw->max-1,
-					MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1),
+					MAX(ptw->drawing->allocation.height/(ptw->fontheight),1),
 																ptw->top.y);
 		set_menu_bar(ptw);
 		draw_caret(ptw,NULL);
@@ -220,6 +232,21 @@ void command_reload(gpointer callback_data,guint callback_action,
 }
 
 
+#ifdef USE_GDKPIXBUF
+void command_print(gpointer callback_data,guint callback_action,
+															GtkWidget *widget)
+{
+	TEXTWND *ptw;
+
+	ptw=gtk_object_get_user_data(GTK_OBJECT(
+				gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
+					gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))));
+	printer.column=ptw->margin;
+	print_dialog(&printer);
+}
+#endif
+
+
 void command_property(gpointer callback_data,guint callback_action,
 															GtkWidget *widget)
 {
@@ -359,11 +386,9 @@ void command_copy(gpointer callback_data,guint callback_action,
 				gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
 					gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))));
 		g_free(clipboard_text);
-		length=get_sel_byte(&ptw->start,&ptw->off,
-										&ptw->select,&ptw->cursor,ptw->tab);
+		length=get_sel_bytes(ptw,&ptw->select,&ptw->cursor);
 		clipboard_text=g_malloc((length+1)*sizeof(gchar));
-		cpy_sel_mem(&ptw->start,&ptw->off,
-							&ptw->select,&ptw->cursor,ptw->tab,clipboard_text);
+		cpy_sel_mem(ptw,&ptw->select,&ptw->cursor,clipboard_text);
 		clipboard_text[length]='\0';
 	}
 }
@@ -475,13 +500,12 @@ void command_all(gpointer callback_data,guint callback_action,
 	if (ptw->max<=1 && ptw->start->length<=0)
 		return;
 	top=ptw->top;
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+	sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 	if (ptw->select.x<0)/* 򤹤 */
 		gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,GDK_CURRENT_TIME);
 	ptw->select.x=ptw->select.y=0;
-	ptw->cursor.x=get_width(&ptw->start,&ptw->off,
-											ptw->cursor.y=ptw->max-1,ptw->tab);
+	ptw->cursor.x=get_width(ptw,ptw->cursor.y=ptw->max-1);
 	if (ptw->cursor.x<ptw->top.x)
 		ptw->top.x=ptw->cursor.x;
 	else if (ptw->cursor.x-sx+1>ptw->top.x)
@@ -644,12 +668,10 @@ void command_replace(gpointer callback_data,guint callback_action,
 			if (ptw->select.x<0)
 				break;
 			replace=TRUE;
-			bytes=get_sel_byte(&ptw->start,&ptw->off,
-										&ptw->select,&ptw->cursor,ptw->tab);
+			bytes=get_sel_bytes(ptw,&ptw->select,&ptw->cursor);
 			if (result<=1) {
 				ri.src=g_malloc(bytes+1);
-				cpy_sel_mem(&ptw->start,&ptw->off,
-									&ptw->select,&ptw->cursor,ptw->tab,ri.src);
+				cpy_sel_mem(ptw,&ptw->select,&ptw->cursor,ri.src);
 				ri.src[bytes]='\0';
 				ri.start=ptw->select;
 				ri.end=ptw->cursor;
@@ -659,15 +681,14 @@ void command_replace(gpointer callback_data,guint callback_action,
 					continue;
 			}
 			if (select.x>=0 && ptw->cursor.y==select.y && replace_arrow)
-				datapos=get_data_pos(&ptw->start,&ptw->off,
-											select.x,select.y,ptw->tab,FALSE);
+				datapos=get_data_pos(ptw,select.x,select.y,FALSE);
 			else
 				datapos=-1;
 			d=edit_operation(ptw,replace_text[0],strlen(replace_text[0]),
 														replace_arrow,TRUE);
 			if (datapos>=0)
-				select.x=get_screen_pos(&ptw->start,&ptw->off,
-							datapos+strlen(rp.dst)-bytes,select.y,ptw->tab);
+				select.x=get_screen_pos(ptw,
+										datapos+strlen(rp.dst)-bytes,select.y);
 			if (result>1)
 				count++;
 			d->next=ptw->undo;
