@@ -46,29 +46,25 @@ LINEBUF *get_line_buf(LINEBUF **start,gint *off,gint y)
 
 
 /*	
-	start,饤Хåե
-	  off,ߤyɸ
-	    y,
-	  tab,
-	  RET,																*/
-gint get_width(LINEBUF **start,gint *off,gint y,gint tab)
+	ptw,TXTɥ
+	  y,
+	RET,																*/
+gint get_width(TEXTWND *ptw,gint y)
 {
 	gint datapos=0,width=0;
 	LINEBUF *p;
 
-	p=get_line_buf(start,off,y);
+	p=get_line_buf(&ptw->start,&ptw->off,y);
 	if (p->margin)
 		while (datapos<p->length-1)
 			if (p->text[datapos]=='\t') {
-				width=(width/tab+1)*tab;
+				width=(width/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																<=p->length) {
-				if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																==p->length)
+			} else if (datapos+charset_length(p->text[datapos])<=p->length) {
+				if (datapos+charset_length(p->text[datapos])==p->length)
 					break;
-				width+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				width+=charset_width(ptw->font,p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			} else {
 				width++;
 				datapos++;
@@ -76,12 +72,11 @@ gint get_width(LINEBUF **start,gint *off,gint y,gint tab)
 	else
 		while (datapos<p->length)
 			if (p->text[datapos]=='\t') {
-				width=(width/tab+1)*tab;
+				width=(width/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																<=p->length) {
-				width+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else if (datapos+charset_length(p->text[datapos])<=p->length) {
+				width+=charset_width(ptw->font,p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			} else {
 				width++;
 				datapos++;
@@ -100,17 +95,16 @@ gint get_width_max(TEXTWND *ptw)
 
 	for (y=0,p=get_line_buf(&ptw->start,&ptw->off,ptw->top.y);
 								y<ptw->drawing->allocation.height && p!=NULL;
-												y+=ptw->fontsize*2,p=p->next) {
+												y+=ptw->fontheight,p=p->next) {
 		width=0;
 		datapos=0;
 		while (datapos<p->length)
 			if (p->text[datapos]=='\t') {
 				width=(width/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																>p->length) {
-				width+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else if (datapos+charset_length(p->text[datapos])>p->length) {
+				width+=charset_width(ptw->font,p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			} else {
 				width++;
 				datapos++;
@@ -123,127 +117,113 @@ gint get_width_max(TEXTWND *ptw)
 
 
 /*	åȤΥ饤Ȥ줿xɸ
-	start,饤Хåե
-	  off,ߤyɸ
+	  ptw,TXTɥ
 	    x,xɸ
 	    y,yɸ
-	  tab,
 	align,TRUE:˴󤻤,FALSE:˴󤻤
 	  RET,ɸ																*/
-gint get_align_pos(LINEBUF **start,gint *off,
-										gint x,gint y,gint tab,gboolean align)
+gint get_align_pos(TEXTWND *ptw,gint x,gint y,gboolean align)
 {
 	gint datapos=0,screenpos=0;
 	LINEBUF *p;
 
-	p=get_line_buf(start,off,y);
+	p=get_line_buf(&ptw->start,&ptw->off,y);
 	if (p->margin)
 		while (screenpos<x && datapos<p->length-1)
 			if (p->text[datapos]=='\t') {
-				if (x<(screenpos/tab+1)*tab)
-					return align?(screenpos/tab+1)*tab:screenpos;
-				screenpos=(screenpos/tab+1)*tab;
+				if (x<(screenpos/ptw->tab+1)*ptw->tab)
+					return align?(screenpos/ptw->tab+1)*ptw->tab:screenpos;
+				screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																>p->length) {
+			} else if (datapos+charset_length(p->text[datapos])>p->length) {
 				screenpos++;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																==p->length) {
+			} else if (datapos+charset_length(p->text[datapos])==p->length) {
 				break;
-			} else if (x<screenpos
-							+charset->width[(gint)(guchar)p->text[datapos]]) {
-				return align
-					?screenpos+charset->width[(gint)(guchar)p->text[datapos]]
-																	:screenpos;
+			} else if (x<screenpos+charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth)) {
+				return align?screenpos+charset_width(ptw->font,
+									p->text+datapos,ptw->fontwidth):screenpos;
 			} else {
-				screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			}
 	else
 		while (screenpos<x && datapos<p->length)
 			if (p->text[datapos]=='\t') {
-				if (x<(screenpos/tab+1)*tab)
-					return align?(screenpos/tab+1)*tab:screenpos;
-				screenpos=(screenpos/tab+1)*tab;
+				if (x<(screenpos/ptw->tab+1)*ptw->tab)
+					return align?(screenpos/ptw->tab+1)*ptw->tab:screenpos;
+				screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																>p->length) {
+			} else if (datapos+charset_length(p->text[datapos])>p->length) {
 				screenpos++;
 				datapos++;
-			} else if (x<screenpos
-							+charset->width[(gint)(guchar)p->text[datapos]]) {
-				return align
-					?screenpos+charset->width[(gint)(guchar)p->text[datapos]]
-																	:screenpos;
+			} else if (x<screenpos+charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth)) {
+				return align?screenpos+charset_width(ptw->font,
+									p->text+datapos,ptw->fontwidth):screenpos;
 			} else {
-				screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			}
 	return screenpos;
 }
 
 
 /*	̾κɸ饤ХåեΥɥ쥹
-	start,饤Хåե
-	  off,ߤyɸ
+	  ptw,TXTɥ
 	    x,xɸ
 	    y,yɸ
-	  tab,
 	align,TRUE:˴󤻤,FALSE:˴󤻤
 	  RET,ɥ쥹															*/
-gint get_data_pos(LINEBUF **start,gint *off,gint x,gint y,
-													gint tab,gboolean align)
+gint get_data_pos(TEXTWND *ptw,gint x,gint y,gboolean align)
 {
 	gint datapos=0,screenpos=0;
 	LINEBUF *p;
 
-	p=get_line_buf(start,off,y);
+	p=get_line_buf(&ptw->start,&ptw->off,y);
 	while (screenpos<x && datapos<p->length)
 		if (p->text[datapos]=='\t') {
-			screenpos=(screenpos/tab+1)*tab;
+			screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 			if (x<screenpos)
 				return align?datapos+1:datapos;
 			datapos++;
-		} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																>p->length) {
+		} else if (datapos+charset_length(p->text[datapos])>p->length) {
 			screenpos++;
 			datapos++;
-		} else if (x<screenpos
-							+charset->width[(gint)(guchar)p->text[datapos]]) {
-			return align
-					?datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																	:datapos;
+		} else if (x<screenpos+charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth)) {
+			return align?datapos+charset_length(p->text[datapos]):datapos;
 		} else {
-			screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-			datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			screenpos+=charset_width(ptw->font,p->text+datapos,ptw->fontwidth);
+			datapos+=charset_length(p->text[datapos]);
 		}
 	return MIN(datapos,p->length);
 }
 
 
 /*	饤ХåեΥɥ쥹̾κɸ
-	start,饤Хåե
-	  off,ߤyɸ
-	    x,xɸ
-	    y,yɸ
-	  tab,
-	  RET,ɸ																*/
-gint get_screen_pos(LINEBUF **start,gint *off,gint x,gint y,gint tab)
+	ptw,TXTɥ
+	  x,xɸ
+	  y,yɸ
+	RET,ɸ																*/
+gint get_screen_pos(TEXTWND *ptw,gint x,gint y)
 {
 	gint datapos=0,screenpos=0;
 	LINEBUF *p;
 
-	p=get_line_buf(start,off,y);
+	p=get_line_buf(&ptw->start,&ptw->off,y);
 	while (datapos<x)
 		if (p->text[datapos]=='\t') {
-			screenpos=(screenpos/tab+1)*tab;
+			screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 			datapos++;
-		} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]>x) {
-			break;
+		} else if (datapos+charset_length(p->text[datapos])<=x) {
+			screenpos+=charset_width(ptw->font,p->text+datapos,ptw->fontwidth);
+			datapos+=charset_length(p->text[datapos]);
 		} else {
-			screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-			datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			break;
 		}
 	return screenpos;
 }
@@ -268,32 +248,28 @@ guint get_char_type(gchar *text,gint length)
 
 
 /*	ڤ
-	start,饤Хåե
-	  off,ߤyɸ
-	    x,xɸ
-	    y,yɸ
-	  tab,
-	 move,TRUE:˰ư,FALSE:˰ư
-	  RET,ɸ																*/
-gint get_move_pos(LINEBUF **start,gint *off,gint x,gint y,
-														gint tab,gboolean move)
+	 ptw,TXTɥ
+	   x,xɸ
+	   y,yɸ
+	move,TRUE:˰ư,FALSE:˰ư
+	 RET,ɸ																*/
+gint get_move_pos(TEXTWND *ptw,gint x,gint y,gboolean move)
 {
 	gint i,j,datapos,length,*type;/* 1ʸΥХȿ */
 	guint ct,ctype;
 	LINEBUF *p;
 
-	p=get_line_buf(start,off,y);
+	p=get_line_buf(&ptw->start,&ptw->off,y);
 	if (move) {
 		/* ذư */
-		datapos=get_data_pos(start,off,x,y,tab,FALSE);
-		length=datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																	<=p->length
-						?charset->length[(gint)(guchar)p->text[datapos]]:1;
+		datapos=get_data_pos(ptw,x,y,FALSE);
+		length=datapos+charset_length(p->text[datapos])<=p->length
+										?charset_length(p->text[datapos]):1;
 		ctype=get_char_type(p->text+datapos,length);
 		datapos+=length;
 		while (datapos<p->length) {
-			length=datapos+charset->length[(gint)(guchar)p->text[datapos]]
-				<=p->length?charset->length[(gint)(guchar)p->text[datapos]]:1;
+			length=datapos+charset_length(p->text[datapos])<=p->length
+										?charset_length(p->text[datapos]):1;
 			ct=get_char_type(p->text+datapos,length);
 			if ((ctype&ct)==0) {
 				if (ct!=0)
@@ -304,16 +280,16 @@ gint get_move_pos(LINEBUF **start,gint *off,gint x,gint y,
 		}
 	} else {
 		/* ذư */
-		datapos=get_data_pos(start,off,x,y,tab,FALSE);
+		datapos=get_data_pos(ptw,x,y,FALSE);
 		if (datapos<=0)
 			return 0;
 		type=g_malloc(datapos*sizeof(gint));
 		i=0;
 		while (i<datapos)
-			if (i+charset->length[(gint)(guchar)p->text[i]]<=datapos) {
-				for (j=0;j<charset->length[(gint)(guchar)p->text[i]];j++)
+			if (i+charset_length(p->text[i])<=datapos) {
+				for (j=0;j<charset_length(p->text[i]);j++)
 					type[i+j]=j+1;
-				i+=charset->length[(gint)(guchar)p->text[i]];
+				i+=charset_length(p->text[i]);
 			} else {
 				type[i++]=1;
 			}
@@ -332,19 +308,16 @@ gint get_move_pos(LINEBUF **start,gint *off,gint x,gint y,
 		}
 		g_free(type);
 	}
-	return get_screen_pos(start,off,datapos,y,tab);
+	return get_screen_pos(ptw,datapos,y);
 }
 
 
 /*	ϰϤΥХȿ
-	  start,饤Хåե
-	    off,ߤyɸ
-	start_p,ϰ
-	  end_p,ϰ
-	    tab,
-	    RET,Хȿ														*/
-gint get_sel_byte(LINEBUF **start,gint *off,
-									GdkPoint *start_p,GdkPoint *end_p,gint tab)
+	     ptw,TXTɥ
+	 start_p,ϰ
+	   end_p,ϰ
+	     RET,Хȿ														*/
+gint get_sel_bytes(TEXTWND *ptw,GdkPoint *start_p,GdkPoint *end_p)
 {
 	gint i,stpos,edpos,length;
 	LINEBUF *p;
@@ -357,14 +330,14 @@ gint get_sel_byte(LINEBUF **start,gint *off,
 		st=*end_p;
 		ed=*start_p;
 	}
-	stpos=get_data_pos(start,off,st.x,st.y,tab,FALSE);
-	edpos=get_data_pos(start,off,ed.x,ed.y,tab,FALSE);
+	stpos=get_data_pos(ptw,st.x,st.y,FALSE);
+	edpos=get_data_pos(ptw,ed.x,ed.y,FALSE);
 	if (st.y==ed.y) {
 		/* Ʊ */
 		length=edpos-stpos;
 	} else {
 		/* 㤦 */
-		p=get_line_buf(start,off,st.y);
+		p=get_line_buf(&ptw->start,&ptw->off,st.y);
 		length=p->length-stpos+edpos+(p->margin?0:1);
 		for (i=st.y+1,p=p->next;i<ed.y && p!=NULL;i++,p=p->next)
 			length+=p->length+(p->margin?0:1);
@@ -374,14 +347,11 @@ gint get_sel_byte(LINEBUF **start,gint *off,
 
 
 /*	ϰϤ˥ԡ
-	  start,饤Хåե
-	    off,ߤyɸ
+	    ptw,TXTɥ
 	start_p,ϰ
 	  end_p,ϰ
-	    tab,
 	   text,															*/
-void cpy_sel_mem(LINEBUF **start,gint *off,
-						GdkPoint *start_p,GdkPoint *end_p,gint tab,gchar *text)
+void cpy_sel_mem(TEXTWND *ptw,GdkPoint *start_p,GdkPoint *end_p,gchar *text)
 {
 	gint i,stpos,edpos,length;
 	LINEBUF *p;
@@ -394,9 +364,9 @@ void cpy_sel_mem(LINEBUF **start,gint *off,
 		st=*end_p;
 		ed=*start_p;
 	}
-	stpos=get_data_pos(start,off,st.x,st.y,tab,FALSE);
-	edpos=get_data_pos(start,off,ed.x,ed.y,tab,FALSE);
-	p=get_line_buf(start,off,st.y);
+	stpos=get_data_pos(ptw,st.x,st.y,FALSE);
+	edpos=get_data_pos(ptw,ed.x,ed.y,FALSE);
+	p=get_line_buf(&ptw->start,&ptw->off,st.y);
 	if (st.y==ed.y) {
 		/* Ʊ */
 		memcpy(text,p->text+stpos,(edpos-stpos)*sizeof(gchar));
@@ -423,30 +393,26 @@ void cpy_sel_mem(LINEBUF **start,gint *off,
 
 
 /*	ƤŽդ
-	 start,饤Хåե
-	   off,ߤyɸ
+	   ptw,TXTɥ
 	   put,Žդɸ
 	   new,ɸ
-	   tab,
 	  text,
 	length,Хȿ
-	margin,ޡ(0ʤޤ֤ʤ)
 	   RET,줿Կ													*/
-gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
-								gint tab,gchar *text,gint length,gint margin)
+gint put_mem(TEXTWND *ptw,GdkPoint *put,GdkPoint *new,gchar *text,gint length)
 {
 	gint i=0,temp_leng,insert=0,datapos,screenpos;
 	LINEBUF *p,*q;
 	gchar *temp;
 
-	datapos=get_data_pos(start,off,put->x,put->y,tab,FALSE);
-	p=get_line_buf(start,off,put->y);
+	datapos=get_data_pos(ptw,put->x,put->y,FALSE);
+	p=get_line_buf(&ptw->start,&ptw->off,put->y);
 	if (datapos<=0 && p->prev!=NULL && p->prev->margin && length>=1
 									&& memcmp(text,"\n",sizeof(gchar))==0) {
 		/* ƬؤιԤޤ֤ƤꡢʸԤΤȤ */
 		p=p->prev;
 		datapos=p->length;
-		put->x=get_screen_pos(start,off,p->length,--put->y,tab);
+		put->x=get_screen_pos(ptw,p->length,--put->y);
 		/* ιԤαüذư */
 	}
 	if (datapos<p->length) {
@@ -459,31 +425,34 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 		temp_leng=0;
 		temp=NULL;
 	}
-	if (margin>0) {
+	if (ptw->limit) {
 		/* ޡޤ֤ */
 		screenpos=0;
 		for (datapos=0;datapos<p->length;datapos++)
 			if (p->text[datapos]=='\t')
-				screenpos=(screenpos/tab+1)*tab;
+				screenpos=(screenpos/ptw->tab+1)*ptw->tab;
+			else if (datapos+charset_length(text[datapos])<=length)
+				screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
 			else
 				screenpos++;
 		while (i<length) {
 			datapos=i;
 			while (datapos<length && text[datapos]!='\n')
 				if (text[datapos]=='\t') {
-					screenpos=(screenpos/tab+1)*tab;
-					if (margin<screenpos)
+					screenpos=(screenpos/ptw->tab+1)*ptw->tab;
+					if (ptw->margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos
-					+charset->length[(gint)(guchar)text[datapos]]<=length) {
-					screenpos+=charset->width[(gint)(guchar)text[datapos]];
-					if (margin<screenpos)
+				} else if (datapos+charset_length(text[datapos])<=length) {
+					screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+					if (ptw->margin<screenpos)
 						break;
-					datapos+=charset->length[(gint)(guchar)text[datapos]];
+					datapos+=charset_length(text[datapos]);
 				} else {
 					screenpos++;
-					if (margin<screenpos)
+					if (ptw->margin<screenpos)
 						break;
 					datapos++;
 				}
@@ -548,53 +517,54 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 		datapos=0;
 		while (datapos<p->length)
 			if (p->text[datapos]=='\t') {
-				new->x=(new->x/tab+1)*tab;
+				new->x=(new->x/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																	<=length) {
-				new->x+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else if (datapos+charset_length(p->text[datapos])<=p->length) {
+				new->x+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			} else {
 				new->x++;
 				datapos++;
 			}
 		new->y=put->y+insert;
-		if (margin>0 && p->margin && (margin<=new->x || (temp!=NULL
-				&& margin<=new->x+charset->width[(gint)(guchar)temp[0]]))) {
+		if (ptw->limit && p->margin && (ptw->margin<=new->x
+				|| (temp!=NULL && ptw->margin<=new->x+charset_width(ptw->font,
+													temp,ptw->fontwidth)))) {
 			new->x=0;
 			new->y++;
 		}
 	}
 	if (temp!=NULL) {
 		/* ְʹߤʸ᤹ */
-		p->text=g_realloc(p->text,p->length+temp_leng);
+		p->text=g_realloc(p->text,(p->length+temp_leng)*sizeof(gchar));
 		memcpy(p->text+p->length,temp,temp_leng*sizeof(gchar));
 		p->length+=temp_leng;
 		g_free(temp);
 	}
-	if (margin>0)
+	if (ptw->limit)
 		/* ޡޤ֤ */
 		while (p!=NULL) {
 			datapos=screenpos=0;
 			while (datapos<p->length)
 				if (p->text[datapos]=='\t') {
-					screenpos=(screenpos/tab+1)*tab;
-					if (margin<screenpos)
+					screenpos=(screenpos/ptw->tab+1)*ptw->tab;
+					if (ptw->margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos
-					+charset->length[(gint)(guchar)text[datapos]]<=length) {
-					screenpos+=charset->width[(gint)(guchar)text[datapos]];
-					if (margin<screenpos)
+				} else if (datapos+charset_length(text[datapos])<=length) {
+					screenpos+=charset_width(ptw->font,
+												text+datapos,ptw->fontwidth);
+					if (ptw->margin<screenpos)
 						break;
-					datapos+=charset->length[(gint)(guchar)text[datapos]];
+					datapos+=charset_length(text[datapos]);
 				} else {
 					screenpos++;
-					if (margin<screenpos)
+					if (ptw->margin<screenpos)
 						break;
 					datapos++;
 				}
-			if (margin<screenpos) {
+			if (ptw->margin<screenpos) {
 				/* ޡĶƤȤԤ */
 				q=g_malloc(sizeof(LINEBUF));
 				q->length=p->length-datapos;
@@ -611,7 +581,7 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 				p->text=g_realloc(p->text,p->length*sizeof(gchar));
 				p=q;
 				insert++;
-			} else if (screenpos<margin && p->margin && p->next!=NULL) {
+			} else if (screenpos<ptw->margin && p->margin && p->next!=NULL) {
 				/* ޡ򲼲굼ԤΤȤιԤȤ碌 */
 				datapos=p->length;
 				q=p->next;
@@ -634,15 +604,11 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 
 
 /*	ϰϤ
-	  start,饤Хåե
-	    off,ߤyɸ
+	    ptw,TXTɥ
 	start_p,ϰ
 	  end_p,ϰ
-	    tab,
-	 margin,ޡ(0ʤޤ֤ʤ)
 	    RET,줿Կ													*/
-gint del_sel_mem(LINEBUF **start,gint *off,
-						GdkPoint *start_p,GdkPoint *end_p,gint tab,gint margin)
+gint del_sel_mem(TEXTWND *ptw,GdkPoint *start_p,GdkPoint *end_p)
 {
 	gint i,stpos,edpos,delete,datapos,screenpos;
 	LINEBUF *p,*q,*r;
@@ -655,9 +621,9 @@ gint del_sel_mem(LINEBUF **start,gint *off,
 		st=*end_p;
 		ed=*start_p;
 	}
-	stpos=get_data_pos(start,off,st.x,st.y,tab,FALSE);
-	edpos=get_data_pos(start,off,ed.x,ed.y,tab,FALSE);
-	p=get_line_buf(start,off,st.y);
+	stpos=get_data_pos(ptw,st.x,st.y,FALSE);
+	edpos=get_data_pos(ptw,ed.x,ed.y,FALSE);
+	p=get_line_buf(&ptw->start,&ptw->off,st.y);
 	if (st.y==ed.y) {
 		/* Ʊ */
 		memcpy(p->text+stpos,p->text+edpos,(p->length-edpos)*sizeof(gchar));
@@ -689,27 +655,27 @@ gint del_sel_mem(LINEBUF **start,gint *off,
 			datapos=screenpos=0;
 			while (datapos<q->length)
 				if (q->text[datapos]=='\t') {
-					screenpos=(screenpos/tab+1)*tab;
+					screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 					datapos++;
-				} else if (datapos
-							+charset->length[(gint)(guchar)q->text[datapos]]
+				} else if (datapos+charset_length(q->text[datapos])
 																<=q->length) {
-					screenpos+=charset->width[(gint)(guchar)q->text[datapos]];
-					datapos+=charset->length[(gint)(guchar)q->text[datapos]];
+					screenpos+=charset_width(ptw->font,
+											q->text+datapos,ptw->fontwidth);
+					datapos+=charset_length(q->text[datapos]);
 				} else {
 					screenpos++;
 					datapos++;
 				}
 			if (p->text[0]=='\t')
-				screenpos=(screenpos/tab+1)*tab;
-			else if (charset->length[(gint)(guchar)p->text[0]]<=p->length)
-				screenpos+=charset->width[(gint)(guchar)p->text[0]];
+				screenpos=(screenpos/ptw->tab+1)*ptw->tab;
+			else if (charset_length(p->text[0])<=p->length)
+				screenpos+=charset_width(ptw->font,p->text,ptw->fontwidth);
 			else
 				screenpos++;
 		} else {
 			screenpos=0;
 		}
-		if (p->length<=0 || screenpos<=margin) {
+		if (p->length<=0 || screenpos<=ptw->margin) {
 			/* ޡ򲼲굼ԤΤȤιԤȤ碌 */
 			datapos=q->length;
 			if (p->next!=NULL)
@@ -733,30 +699,29 @@ gint del_sel_mem(LINEBUF **start,gint *off,
 			}
 		}
 	}
-	if (margin>0)
-		/* ޡޤ֤ */
+	if (ptw->limit)/* ޡޤ֤ */
 		while (p!=NULL) {
 			datapos=screenpos=0;
 			while (datapos<p->length)
 				if (p->text[datapos]=='\t') {
-					screenpos=(screenpos/tab+1)*tab;
-					if (margin<screenpos)
+					screenpos=(screenpos/ptw->tab+1)*ptw->tab;
+					if (ptw->margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos
-							+charset->length[(gint)(guchar)p->text[datapos]]
+				} else if (datapos+charset_length(p->text[datapos])
 																<=p->length) {
-					screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-					if (margin<screenpos)
+					screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+					if (ptw->margin<screenpos)
 						break;
-					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+					datapos+=charset_length(p->text[datapos]);
 				} else {
 					screenpos++;
-					if (margin<screenpos)
+					if (ptw->margin<screenpos)
 						break;
 					datapos++;
 				}
-			if (margin<screenpos) {
+			if (ptw->margin<screenpos) {
 				/* ޡĶƤȤԤ */
 				q=g_malloc(sizeof(LINEBUF));
 				q->length=p->length-datapos;
@@ -773,7 +738,7 @@ gint del_sel_mem(LINEBUF **start,gint *off,
 				p->text=g_realloc(p->text,p->length*sizeof(gchar));
 				p=q;
 				delete--;
-			} else if (screenpos<margin && p->margin && p->next!=NULL) {
+			} else if (screenpos<ptw->margin && p->margin && p->next!=NULL) {
 				/* ޡ򲼲굼ԤΤȤιԤȤ碌 */
 				datapos=p->length;
 				q=p->next;
