@@ -43,13 +43,12 @@ void hide_caret(TEXTWND *ptw)
 	timer_id=0;
 	if (caret) {
 		caret=FALSE;
-		rc.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+		rc.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 		rc.y=ptw->cursor.y;
-		rc.x=(rc.x-ptw->top.x)*ptw->fontsize;
-		rc.y=(rc.y-ptw->top.y)*ptw->fontsize*2;
-		rc.width=ptw->fontsize;
-		rc.height=ptw->fontsize*2;
+		rc.x=(rc.x-ptw->top.x)*ptw->fontwidth;
+		rc.y=(rc.y-ptw->top.y)*ptw->fontheight;
+		rc.width=ptw->fontwidth;
+		rc.height=ptw->fontheight;
 		gtk_widget_draw(ptw->drawing,&rc);
 	}
 }
@@ -71,25 +70,23 @@ void draw_caret(TEXTWND *ptw,GdkPoint *cursor)
 		gtk_timeout_remove(timer_id);
 		timer_id=0;
 		/* ŤåȤõ */
-		rc.x=get_align_pos(&ptw->start,&ptw->off,
-										cursor->x,cursor->y,ptw->tab,FALSE);
+		rc.x=get_align_pos(ptw,cursor->x,cursor->y,FALSE);
 		rc.y=cursor->y;
-		rc.x=(rc.x-ptw->top.x)*ptw->fontsize;
-		rc.y=(rc.y-ptw->top.y)*ptw->fontsize*2;
-		rc.width=ptw->fontsize;
-		rc.height=ptw->fontsize*2;
+		rc.x=(rc.x-ptw->top.x)*ptw->fontwidth;
+		rc.y=(rc.y-ptw->top.y)*ptw->fontheight;
+		rc.width=ptw->fontwidth;
+		rc.height=ptw->fontheight;
 		gtk_widget_draw(ptw->drawing,&rc);
 	}
 	/* åȤ */
 	if (timer_id==0)
 		caret=TRUE;
-	x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+	x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 	y=ptw->cursor.y;
-	rc.x=(x-ptw->top.x)*ptw->fontsize;
-	rc.y=(y-ptw->top.y)*ptw->fontsize*2;
-	rc.width=ptw->fontsize;
-	rc.height=ptw->fontsize*2;
+	rc.x=(x-ptw->top.x)*ptw->fontwidth;
+	rc.y=(y-ptw->top.y)*ptw->fontheight;
+	rc.width=ptw->fontwidth;
+	rc.height=ptw->fontheight;
 	gtk_widget_draw(ptw->drawing,&rc);
 	if (timer_id==0)
 		timer_id=gtk_timeout_add(500,signal_timeout,NULL);
@@ -134,6 +131,10 @@ void set_menu_bar(TEXTWND *ptw)
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				_("<main>/ե(F)/¸Ƥ֤᤹(R)...")),
 																!ptw->create);
+#ifdef USE_GDKPIXBUF
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
+				_("<main>/ե(F)/(P)...")),TRUE);
+#endif
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				_("<main>/ե(F)/ץѥƥ(I)...")),TRUE);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
@@ -152,6 +153,17 @@ void set_menu_bar(TEXTWND *ptw)
 				_("<main>/(S)/դ򸡺(B)")),find_num>0);
 		gtk_widget_set_sensitive(toolbar_items[ 2].item,TRUE);
 		gtk_widget_set_sensitive(toolbar_items[ 4].item,TRUE);
+#ifdef USE_GDKPIXBUF
+		gtk_widget_set_sensitive(toolbar_items[ 6].item,TRUE);
+		gtk_widget_set_sensitive(toolbar_items[ 8].item,ptw->select.x>=0);
+		gtk_widget_set_sensitive(toolbar_items[ 9].item,ptw->select.x>=0);
+		gtk_widget_set_sensitive(toolbar_items[10].item,TRUE);
+		gtk_widget_set_sensitive(toolbar_items[11].item,ptw->select.x>=0);
+		gtk_widget_set_sensitive(toolbar_items[13].item,ptw->undo!=NULL);
+		gtk_widget_set_sensitive(toolbar_items[14].item,ptw->redo!=NULL);
+		gtk_widget_set_sensitive(toolbar_items[16].item,TRUE);
+		gtk_widget_set_sensitive(toolbar_items[17].item,TRUE);
+#else
 		gtk_widget_set_sensitive(toolbar_items[ 6].item,ptw->select.x>=0);
 		gtk_widget_set_sensitive(toolbar_items[ 7].item,ptw->select.x>=0);
 		gtk_widget_set_sensitive(toolbar_items[ 8].item,TRUE);
@@ -160,7 +172,7 @@ void set_menu_bar(TEXTWND *ptw)
 		gtk_widget_set_sensitive(toolbar_items[12].item,ptw->redo!=NULL);
 		gtk_widget_set_sensitive(toolbar_items[14].item,TRUE);
 		gtk_widget_set_sensitive(toolbar_items[15].item,TRUE);
-		gtk_widget_set_sensitive(toolbar_items[17].item,TRUE);
+#endif
 	} else {
 		gtk_widget_set_sensitive(gtk_menu_get_attach_widget(GTK_MENU(
 								gtk_item_factory_get_widget(item_factory_menu,
@@ -176,10 +188,25 @@ void set_menu_bar(TEXTWND *ptw)
 				_("<main>/ե(F)/̾Ĥ¸(A)...")),FALSE);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				_("<main>/ե(F)/¸Ƥ֤᤹(R)...")),FALSE);
+#ifdef USE_GDKPIXBUF
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
+				_("<main>/ե(F)/(P)...")),FALSE);
+#endif
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				_("<main>/ե(F)/ץѥƥ(I)...")),FALSE);
 		gtk_widget_set_sensitive(toolbar_items[ 2].item,FALSE);
 		gtk_widget_set_sensitive(toolbar_items[ 4].item,FALSE);
+#ifdef USE_GDKPIXBUF
+		gtk_widget_set_sensitive(toolbar_items[ 6].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[ 8].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[ 9].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[10].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[11].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[13].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[14].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[16].item,FALSE);
+		gtk_widget_set_sensitive(toolbar_items[17].item,FALSE);
+#else
 		gtk_widget_set_sensitive(toolbar_items[ 6].item,FALSE);
 		gtk_widget_set_sensitive(toolbar_items[ 7].item,FALSE);
 		gtk_widget_set_sensitive(toolbar_items[ 8].item,FALSE);
@@ -188,7 +215,7 @@ void set_menu_bar(TEXTWND *ptw)
 		gtk_widget_set_sensitive(toolbar_items[12].item,FALSE);
 		gtk_widget_set_sensitive(toolbar_items[14].item,FALSE);
 		gtk_widget_set_sensitive(toolbar_items[15].item,FALSE);
-		gtk_widget_set_sensitive(toolbar_items[17].item,FALSE);
+#endif
 	}
 	menu=gtk_item_factory_get_widget(item_factory_menu,
 													_("<main>/ɥ(W)"));
@@ -210,20 +237,18 @@ void clear_sel(TEXTWND *ptw,GdkPoint *start,GdkPoint *end)
 
 	if (start->y==end->y) {
 		/* Ʊ */
-		st=get_align_pos(&ptw->start,&ptw->off,
-								MIN(start->x,end->x),start->y,ptw->tab,FALSE);
-		ed=get_align_pos(&ptw->start,&ptw->off,
-								MAX(start->x,end->x),start->y,ptw->tab,TRUE);
-		rc.x=(st-ptw->top.x)*ptw->fontsize;
-		rc.y=(start->y-ptw->top.y)*ptw->fontsize*2;
-		rc.width=(ed-st+1)*ptw->fontsize;
-		rc.height=ptw->fontsize*2;
+		st=get_align_pos(ptw,MIN(start->x,end->x),start->y,FALSE);
+		ed=get_align_pos(ptw,MAX(start->x,end->x),start->y,TRUE);
+		rc.x=(st-ptw->top.x)*ptw->fontwidth;
+		rc.y=(start->y-ptw->top.y)*ptw->fontheight;
+		rc.width=(ed-st+1)*ptw->fontwidth;
+		rc.height=ptw->fontheight;
 	} else {
 		/* 㤦 */
 		rc.x=0;
-		rc.y=(MIN(start->y,end->y)-ptw->top.y)*ptw->fontsize*2;
+		rc.y=(MIN(start->y,end->y)-ptw->top.y)*ptw->fontheight;
 		rc.width=ptw->drawing->allocation.width;
-		rc.height=(ABS(start->y-end->y)+1)*ptw->fontsize*2;
+		rc.height=(ABS(start->y-end->y)+1)*ptw->fontheight;
 	}
 	gtk_widget_draw(ptw->drawing,&rc);
 }
@@ -236,15 +261,15 @@ void move_text_window(TEXTWND *ptw,GdkPoint *top)
 {
 	gint sx,sy;
 
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+	sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 	if (ptw->top.x!=top->x || ptw->top.y!=top->y) {
 		set_scroll_bar(ptw->hscroll,signal_value_changed_hscroll,ptw,
 										0,get_width_max(ptw),sx,ptw->top.x);
 		set_scroll_bar(ptw->vscroll,signal_value_changed_vscroll,ptw,
 										0,ptw->max-1,sy,ptw->top.y);
-		scroll_window(ptw->drawing,	(top->x-ptw->top.x)*ptw->fontsize,
-									(top->y-ptw->top.y)*ptw->fontsize*2,
+		scroll_window(ptw->drawing,	(top->x-ptw->top.x)*ptw->fontwidth,
+									(top->y-ptw->top.y)*ptw->fontheight,
 									0,0,
 									ptw->drawing->allocation.width,
 									ptw->drawing->allocation.height);
@@ -269,11 +294,10 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 	GdkRectangle rc;
 
 	hide_caret(ptw);
-	ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+	ptw->cursor.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 	top=ptw->top;
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+	sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 	d=g_malloc(sizeof(DOING));
 	if (ptw->select.x<0) {
 		d->text=NULL;
@@ -281,13 +305,10 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 		d->caret=FALSE;
 	} else {
 		/*  */
-		d->length=get_sel_byte(&ptw->start,&ptw->off,
-										&ptw->cursor,&ptw->select,ptw->tab);
+		d->length=get_sel_bytes(ptw,&ptw->cursor,&ptw->select);
 		d->text=g_malloc(d->length*sizeof(gchar));
-		cpy_sel_mem(&ptw->start,&ptw->off,
-								&ptw->cursor,&ptw->select,ptw->tab,d->text);
-		delete=del_sel_mem(&ptw->start,&ptw->off,
-				&ptw->cursor,&ptw->select,ptw->tab,ptw->limit?ptw->margin:0);
+		cpy_sel_mem(ptw,&ptw->cursor,&ptw->select,d->text);
+		delete=del_sel_mem(ptw,&ptw->cursor,&ptw->select);
 		ptw->max-=delete;/* Կ */
 		/* ϰϲȥåȰư */
 		if (ptw->select.y<ptw->cursor.y || (ptw->select.y==ptw->cursor.y
@@ -300,18 +321,18 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 		}
 		ptw->select.x=-1;
 		/* Ԥκ걦(ޡΤ1¿) */
-		rc.x=(ptw->cursor.x-ptw->top.x-1)*ptw->fontsize;
-		rc.y=(ptw->cursor.y-ptw->top.y)*ptw->fontsize*2;
+		rc.x=(ptw->cursor.x-ptw->top.x-1)*ptw->fontwidth;
+		rc.y=(ptw->cursor.y-ptw->top.y)*ptw->fontheight;
 		rc.width=ptw->drawing->allocation.width-rc.x;
-		rc.height=ptw->fontsize*2;
+		rc.height=ptw->fontheight;
 		gtk_widget_draw(ptw->drawing,&rc);
 		/* äԿʬ */
-		scroll_window(ptw->drawing,0,-delete*ptw->fontsize*2,
+		scroll_window(ptw->drawing,0,-delete*ptw->fontheight,
 						0,
-						(ptw->cursor.y-ptw->top.y+1)*ptw->fontsize*2,
+						(ptw->cursor.y-ptw->top.y+1)*ptw->fontheight,
 						ptw->drawing->allocation.width,
 						MAX(ptw->drawing->allocation.height
-							-(ptw->cursor.y-ptw->top.y+1)*ptw->fontsize*2,0));
+							-(ptw->cursor.y-ptw->top.y+1)*ptw->fontheight,0));
 		/* 򽪤֤鱦ޡαƶΤԿ */
 		for (i=0,p=get_line_buf(&ptw->start,&ptw->off,ptw->cursor.y);
 														p!=NULL;i++,p=p->next)
@@ -319,9 +340,9 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 				break;
 		if (i>0) {
 			rc.x=0;
-			rc.y=(ptw->cursor.y-ptw->top.y+1)*ptw->fontsize*2;
+			rc.y=(ptw->cursor.y-ptw->top.y+1)*ptw->fontheight;
 			rc.width=ptw->drawing->allocation.width;
-			rc.height=i*ptw->fontsize*2;
+			rc.height=i*ptw->fontheight;
 			gtk_widget_draw(ptw->drawing,&rc);
 		}
 	}
@@ -330,10 +351,8 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 		d->select.x=-1;
 	} else {
 		/*  */
-		ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
-		put=put_mem(&ptw->start,&ptw->off,&ptw->cursor,&cursor_new,
-								ptw->tab,text,length,ptw->limit?ptw->margin:0);
+		ptw->cursor.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
+		put=put_mem(ptw,&ptw->cursor,&cursor_new,text,length);
 		ptw->max+=put;/* Կ */
 		cursor_old=ptw->cursor;
 		if (caret) {
@@ -349,22 +368,22 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 		if (select) {
 			/* 򤹤 */
 			ptw->select=d->select;
-			gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+			gtk_selection_owner_set(ptw->drawing,GDK_SELECTION_PRIMARY,
 															GDK_CURRENT_TIME);
 		}
 		/* Ԥ걦 */
-		rc.x=(cursor_old.x-ptw->top.x)*ptw->fontsize;
-		rc.y=(cursor_old.y-ptw->top.y)*ptw->fontsize*2;
+		rc.x=(cursor_old.x-ptw->top.x)*ptw->fontwidth;
+		rc.y=(cursor_old.y-ptw->top.y)*ptw->fontheight;
 		rc.width=ptw->drawing->allocation.width;
-		rc.height=ptw->fontsize*2;
+		rc.height=ptw->fontheight;
 		gtk_widget_draw(ptw->drawing,&rc);
 		/* Կʬ */
-		scroll_window(ptw->drawing,0,put*ptw->fontsize*2,
+		scroll_window(ptw->drawing,0,put*ptw->fontheight,
 						0,
-						(cursor_old.y-ptw->top.y+1)*ptw->fontsize*2,
+						(cursor_old.y-ptw->top.y+1)*ptw->fontheight,
 						ptw->drawing->allocation.width,
 						MAX(ptw->drawing->allocation.height
-							-(cursor_old.y-ptw->top.y+1)*ptw->fontsize*2,0));
+							-(cursor_old.y-ptw->top.y+1)*ptw->fontheight,0));
 		/* 򽪤֤鱦ޡαƶΤԿ */
 		p=q=get_line_buf(&ptw->start,&ptw->off,cursor_new.y);
 		for (i=cursor_new.y,j=0;
@@ -374,9 +393,9 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 			j++,q=q->next;
 		if (j>0) {
 			rc.x=0;
-			rc.y=(i-ptw->top.y+1)*ptw->fontsize*2;
+			rc.y=(i-ptw->top.y+1)*ptw->fontheight;
 			rc.width=ptw->drawing->allocation.width;
-			rc.height=j*ptw->fontsize*2;
+			rc.height=j*ptw->fontheight;
 			gtk_widget_draw(ptw->drawing,&rc);
 		}
 	}
@@ -444,12 +463,13 @@ void modify_margin(TEXTWND *ptw)
 					if (ptw->margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos+charset->length[(gint)(guchar)
-												p->text[datapos]]<=p->length) {
-					screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+				} else if (datapos+charset_length(p->text[datapos])
+																<=p->length) {
+					screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
 					if (ptw->margin<screenpos)
 						break;
-					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+					datapos+=charset_length(p->text[datapos]);
 				} else {
 					screenpos++;
 					if (ptw->margin<screenpos)
@@ -520,7 +540,7 @@ void modify_margin(TEXTWND *ptw)
 void change_property(TEXTWND *ptw,FTYPE *ft)
 {
 	gboolean modify;
-	gint i,max,sx,sy,fontsize,ascent;
+	gint max,sx,sy;
 
 	if (ptw->limit!=ft->limit || ptw->tab!=ft->tab
 								|| (ptw->code && ptw->margin!=ft->margin)) {
@@ -529,7 +549,8 @@ void change_property(TEXTWND *ptw,FTYPE *ft)
 		delete_list(&ptw->redo);
 	}
 	modify=ptw->limit!=ft->limit || (ptw->limit
-							&& (ptw->tab!=ft->tab || ptw->margin!=ft->margin));
+							&& (ptw->tab!=ft->tab || ptw->margin!=ft->margin
+									|| strcmp(ptw->fontname,ft->fontname)!=0));
 	ptw->ft_id=ft->ft_id;
 	ptw->margin=ft->margin;
 	ptw->tab=ft->tab;
@@ -548,29 +569,23 @@ void change_property(TEXTWND *ptw,FTYPE *ft)
 	ptw->fontname=g_strdup(ft->fontname);
 	memcpy(ptw->color,ft->color,sizeof(GdkColor)*12);
 	ptw->select.x=-1;
-	if (modify) {
-		/* ޤ֤ѹޤ֤ͭǥֿѹ */
-		ptw->cursor.x=ptw->cursor.y=ptw->top.x=ptw->top.y=0;
-		modify_margin(ptw);
-	}
 	/* ե */
 	if (ptw->fontname!=NULL) {
 		gdk_font_unref(ptw->font);
-		ptw->font=gdk_fontset_load(ptw->fontname);
-		ptw->fontascent=ptw->fontsize=0;
-		for (i=0x20;i<=0x7e;i++) {
-			gdk_text_extents(ptw->font,(gchar *)&i,1,
-												NULL,NULL,NULL,&ascent,NULL);
-			if (ptw->fontascent<ascent)
-				ptw->fontascent=ascent;
-			fontsize=(gdk_char_height(ptw->font,i)+1)/2;
-			if (ptw->fontsize<fontsize)
-				ptw->fontsize=fontsize;
-		}
+		if ((ptw->font=gdk_fontset_load(ptw->fontname))==NULL)
+			ptw->font=gdk_font_ref(system_font);
+		charset_size(ptw->font,
+							&ptw->fontwidth,&ptw->fontheight,&ptw->fontascent);
+	}
+	if (modify) {
+		/* ޤ֤ѹ
+							ޤ֤ͭǥֿ䱦ޡեѹ */
+		ptw->cursor.x=ptw->cursor.y=ptw->top.x=ptw->top.y=0;
+		modify_margin(ptw);
 	}
 	/* С */
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+	sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 	if (ptw->top.y>ptw->max-sy)
 		ptw->top.y=MAX(ptw->max-sy,0);
 	max=get_width_max(ptw);
@@ -691,10 +706,10 @@ void margin_operation(TEXTWND *ptw)
 	ptw->select.x=-1;
 	set_scroll_bar(ptw->hscroll,signal_value_changed_hscroll,ptw,
 		0,get_width_max(ptw),
-		MAX(ptw->drawing->allocation.width/ptw->fontsize,1),ptw->top.x);
+		MAX(ptw->drawing->allocation.width/ptw->fontwidth,1),ptw->top.x);
 	set_scroll_bar(ptw->vscroll,signal_value_changed_vscroll,ptw,
 		0,ptw->max-1,
-		MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1),ptw->top.y);
+		MAX(ptw->drawing->allocation.height/ptw->fontheight,1),ptw->top.y);
 	set_menu_bar(ptw);
 	gtk_widget_draw(ptw->drawing,NULL);
 }
@@ -717,10 +732,10 @@ void tab_operation(TEXTWND *ptw)
 			if (p->text[datapos]=='\t') {
 				screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																<=p->length) {
-				screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else if (datapos+charset_length(p->text[datapos])<=p->length) {
+				screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+				datapos+=charset_length(p->text[datapos]);
 			} else {
 				screenpos++;
 				datapos++;
@@ -733,12 +748,13 @@ void tab_operation(TEXTWND *ptw)
 				if (p->text[datapos]=='\t') {
 					screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 					datapos++;
-				} else if (datapos+charset->length[(gint)(guchar)
-												p->text[datapos]]<=p->length) {
+				} else if (datapos+charset_length(p->text[datapos])
+																<=p->length) {
 					memmove(temp+screenpos,p->text+datapos,
-							charset->length[(gint)(guchar)p->text[datapos]]);
-					screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
-					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+											charset_length(p->text[datapos]));
+					screenpos+=charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+					datapos+=charset_length(p->text[datapos]);
 				} else {
 					temp[screenpos]=p->text[datapos];
 					screenpos++;
