@@ -39,8 +39,8 @@ void jump_operation(TEXTWND *ptw,gint line)
 	cursor=ptw->cursor;
 	select=ptw->select;
 	top=ptw->top;
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+	sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 	ptw->cursor.y=line<0?ptw->max+line:line-1;
 	if (ptw->cursor.y<0)
 		ptw->cursor.y=0;
@@ -57,8 +57,8 @@ void jump_operation(TEXTWND *ptw,gint line)
 	move_text_window(ptw,&top);
 	if (ptw->select.x>=0) {
 		if (ptw->select.y==ptw->cursor.y
-						&& ptw->select.x==get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE)) {
+									&& ptw->select.x==get_align_pos(ptw,
+										ptw->cursor.x,ptw->cursor.y,FALSE)) {
 			ptw->select.x=-1;
 			clear_sel(ptw,&select,&cursor);
 			set_menu_bar(ptw);
