@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <gdk/gdkkeysyms.h>
+#include "charset.h"
 #include "other.h"
 #include "valchr.h"
 
@@ -39,7 +40,7 @@ static void valchr_dialog_changed(GtkWidget *widget,gpointer user_data)
 
 	strval(&value,gtk_entry_get_text(GTK_ENTRY(widget)),valchr_base,FALSE);
 	gtk_widget_set_sensitive(button0,0<=value && value<=255
-														&& !iskanji(value));
+												&& charset->length[value]==1);
 }
 
 
@@ -75,7 +76,7 @@ static void valchr_dialog_activate(GtkWidget *widget,gchar *valchr)
 	gint value;
 
 	strval(&value,gtk_entry_get_text(GTK_ENTRY(widget)),valchr_base,FALSE);
-	if (0<=value && value<=255 && !iskanji(value))
+	if (0<=value && value<=255 && charset->length[value]==1)
 		valchr_dialog_clicked_ok(widget,valchr);
 }
 
